@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2023 Intel Corporation.
+ *   Copyright(c) 2016-2024 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -164,13 +164,13 @@ int qat_fips_kat_test;
 const char *engine_qat_id = STR(QAT_ENGINE_ID);
 #if defined(QAT_HW) && defined(QAT_SW)
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v1.5.0";
+    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v1.6.1";
 #elif QAT_HW
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw) v1.5.0";
+    "Reference implementation of QAT crypto engine(qat_hw) v1.6.1";
 #else
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_sw) v1.5.0";
+    "Reference implementation of QAT crypto engine(qat_sw) v1.6.1";
 #endif
 unsigned int engine_inited = 0;
 int fallback_to_openssl = 0;
@@ -243,7 +243,11 @@ int qat_cond_wait_started = 0;
 
 #ifdef QAT_HW
 # define QAT_CONFIG_SECTION_NAME_SIZE 64
+# if defined(QAT_HW_FBSD_INTREE)
+char qat_config_section_name[QAT_CONFIG_SECTION_NAME_SIZE] = "SSL";
+# else
 char qat_config_section_name[QAT_CONFIG_SECTION_NAME_SIZE] = "SHIM";
+# endif
 char *ICPConfigSectionName_libcrypto = qat_config_section_name;
 int enable_inline_polling = 0;
 int enable_event_driven_polling = 0;
@@ -603,6 +607,7 @@ int qat_engine_init(ENGINE *e)
         if (!qat_hw_init(e)) {
 # ifdef ENABLE_QAT_FIPS
             fprintf(stderr, "QAT_HW initialization Failed\n");
+            qat_pthread_mutex_unlock();
             return 0;
 # else
 #  ifdef QAT_SW /* Co-Existence mode: Don't return failure when QAT HW initialization Failed. */
@@ -623,6 +628,7 @@ int qat_engine_init(ENGINE *e)
         if (!qat_sw_init(e)) {
 # ifdef ENABLE_QAT_FIPS
             fprintf(stderr, "QAT_SW initialization Failed\n");
+            qat_pthread_mutex_unlock();
             return 0;
 # else
             WARN("QAT SW initialization Failed, switching to OpenSSL.\n");
@@ -753,14 +759,15 @@ int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
             BREAK_IF(!enable_external_polling, "POLL failed as external polling is not enabled\n");
             BREAK_IF(p == NULL, "POLL failed as the input parameter was NULL\n");
 #ifdef QAT_HW
-            if (qat_hw_offload) {
+            if (qat_hw_offload && !fallback_to_qat_sw) {
                 BREAK_IF(qat_instance_handles == NULL, "POLL failed as no instances are available\n");
                 *(int *)p = (int)poll_instances();
             }
 #endif
 
 #ifdef QAT_SW
-            *(int *)p = qat_sw_poll();
+            if (qat_sw_offload)
+                *(int *)p = qat_sw_poll();
 #endif
         break;
 
@@ -978,14 +985,17 @@ int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
         case QAT_CMD_HEARTBEAT_POLL:
 # if !defined(__FreeBSD__) && !defined(QAT_HW_INTREE)
         BREAK_IF(!engine_inited, "HEARTBEAT_POLL failed as engine is not initialized\n");
-        BREAK_IF(qat_instance_handles == NULL,
-                "HEARTBEAT_POLL failed as no instances are available\n");
         BREAK_IF(!enable_external_polling,
                 "HEARTBEAT_POLL failed as external polling is not enabled\n");
         BREAK_IF(p == NULL, "HEARTBEAT_POLL failed as the input parameter was NULL\n");
 
-        *(int *)p = (int)poll_heartbeat();
-        CRYPTO_QAT_LOG("QAT Engine Heartbeat Poll - %s\n", __func__);
+        if (qat_instance_handles != NULL) {
+            *(int *)p = (int)poll_heartbeat();
+            CRYPTO_QAT_LOG("QAT Engine Heartbeat Poll - %s\n", __func__);
+        } else if (!fallback_to_qat_sw) {
+            WARN("HEARTBEAT_POLL failed as no instances are available\n");
+            retVal = 0;
+        }
 # else
         WARN("QAT_CMD_HEARTBEAT_POLL is not supported\n");
         retVal = 0;
@@ -1368,6 +1378,10 @@ int bind_qat(ENGINE *e, const char *id)
         qat_sw_sm3_offload = 1;
         INFO("QAT_SW SM3 for Provider Enabled\n");
 # endif
+# ifdef ENABLE_QAT_HW_CCM
+        qat_hw_aes_ccm_offload = 1;
+        INFO("QAT_HW AES-CCM for Provider Enabled\n");
+# endif
 #endif
 
 #ifndef QAT_BORINGSSL
