@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2020-2023 Intel Corporation.
+ *   Copyright(c) 2020-2024 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -49,12 +49,6 @@
 #include <string.h>
 #include <pthread.h>
 #include <signal.h>
-#ifdef USE_QAT_CONTIG_MEM
-# include "qae_mem_utils.h"
-#endif
-#ifdef USE_USDM_MEM
-# include "qat_hw_usdm_inf.h"
-#endif
 #include "e_qat.h"
 #include "qat_utils.h"
 #include "qat_hw_callback.h"
@@ -368,9 +362,10 @@ int qat_aes_gcm_init(EVP_CIPHER_CTX *ctx,
 
 err:
     if (NULL != qctx->iv) {
-        if (qctx->iv != EVP_CIPHER_CTX_iv_noconst(ctx))
+        if (qctx->iv != EVP_CIPHER_CTX_iv_noconst(ctx)) {
             QAT_MEM_FREE_NONZERO_BUFF(qctx->iv, qctx->qat_svm);
-        qctx->iv = NULL;
+            qctx->iv = NULL;
+	}
     }
 
     QAT_MEM_FREE_BUFF(qctx->cipher_key, qctx->qat_svm);
@@ -803,6 +798,8 @@ int qat_aes_gcm_cleanup(EVP_CIPHER_CTX *ctx)
 
             /* Cleanup the memory */
             QAT_MEM_FREE_NONZERO_BUFF(qctx->qat_ctx, qctx->qat_svm);
+            if (!qctx->qat_svm)
+                QAT_MEM_FREE_NONZERO_BUFF(qctx->aad, qctx->qat_svm);
             QAT_MEM_FREE_NONZERO_BUFF(qctx->srcBufferList.pPrivateMetaData, qctx->qat_svm);
             QAT_MEM_FREE_NONZERO_BUFF(qctx->dstBufferList.pPrivateMetaData, qctx->qat_svm);
             QAT_MEM_FREE_NONZERO_BUFF(qctx->iv, qctx->qat_svm);
@@ -1726,8 +1723,10 @@ int qat_aes_gcm_cipher(EVP_CIPHER_CTX *ctx, unsigned char *out,
     } else {
         /* This is executed when Final is called */
         if (!enc) {
+# if OPENSSL_VERSION_NUMBER < 0x30200000
             if (qctx->tag_len < 0)
                 return RET_FAIL;
+# endif
             /* Don't reuse the IV */
             qctx->iv_set = 0;
             DEBUG("Decrypt Final()\n");
