@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023 Intel Corporation.
+ *   Copyright(c) 2023-2024 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@
 #define __USE_GNU
 #include "qat_hw_sm2.h"
 
-#ifdef ENABLE_QAT_HW_SM2
+#if defined(ENABLE_QAT_HW_SM2) && !defined(QAT_HW_FBSD_INTREE)
 typedef struct {
     /* Key and paramgen group */
     EC_GROUP *gen_group;
@@ -704,7 +704,7 @@ int qat_sm2_sign(EVP_PKEY_CTX *ctx,
             goto err;
         } else {
             QATerr(QAT_F_QAT_SM2_SIGN, ERR_R_INTERNAL_ERROR);
-	    return 0;
+	    goto err;
         }
     }
     qat_svm = !qat_instance_details[inst_num].qat_instance_info.requiresPhysicallyContiguousMemory;
@@ -970,6 +970,9 @@ int qat_sm2_sign(EVP_PKEY_CTX *ctx,
         BN_CTX_free(bctx);
     }
 
+    BN_free(sig_r);
+    BN_free(sig_s);
+
     if (fallback) {
         WARN("- Fallback to software mode.\n");
         CRYPTO_QAT_LOG("Resubmitting request to SW - %s\n", __func__);
