@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023 Intel Corporation.
+ *   Copyright(c) 2023-2024 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -49,12 +49,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <signal.h>
-#ifdef USE_QAT_CONTIG_MEM
-# include "qae_mem_utils.h"
-#endif
-#ifdef USE_USDM_MEM
-# include "qat_hw_usdm_inf.h"
-#endif
+
 #include "e_qat.h"
 #include "qat_utils.h"
 #include "qat_hw_callback.h"
@@ -75,7 +70,6 @@
 #include <openssl/ssl.h>
 
 #ifdef ENABLE_QAT_HW_SM3
-
 static inline QAT_SM3_CTX *qat_hw_sm3_get_ctx(EVP_MD_CTX *ctx)
 {
     if (unlikely(ctx == NULL)) {
@@ -361,7 +355,7 @@ static int qat_hw_sm3_do_offload(QAT_SM3_CTX *qat_sm3_ctx, const void *in,
                 ("Failed to submit request to qat inst_num %d device_id %d - %s\n",
                  qat_sm3_ctx->inst_num,
                  qat_instance_details[qat_sm3_ctx->inst_num].qat_instance_info.
-                 physInstId.packageId);
+                 physInstId.packageId, __func__);
         }
         QATerr(QAT_F_QAT_HW_SM3_DO_OFFLOAD, ERR_R_INTERNAL_ERROR);
         if (op_done.job != NULL)
@@ -418,7 +412,7 @@ static int qat_hw_sm3_do_offload(QAT_SM3_CTX *qat_sm3_ctx, const void *in,
                 ("Verification of result failed for qat inst_num %d device_id %d - %s\n",
                  inst_num,
                  qat_instance_details[qat_sm3_ctx->_inst_num].qat_instance_info.
-                 physInstId.packageId);
+                 physInstId.packageId, __func__);
             qat_cleanup_op_done(&op_done);
             goto err;
         }
@@ -811,7 +805,7 @@ int qat_hw_sm3_cleanup(EVP_MD_CTX *ctx)
         } else {
             OPENSSL_free(qat_sm3_ctx->data);
             OPENSSL_free(qat_sm3_ctx->data_refs);
-        }
+	}
     }
 
     if (qat_sm3_ctx->context_params_set) {
@@ -916,7 +910,11 @@ const EVP_MD *qat_hw_create_sm3_meth(int nid, int key_type)
         }
         return qat_hw_sm3_meth;
 # else
+#  ifdef OPENSSL_NO_SM2_SM3
+	return NULL;
+#  else
         return (EVP_MD *)EVP_sm3();
+#  endif
 # endif
     }
 }
